localStorage.clear();
ProjectMaya = new Class({

	Implements: [Events, Options],

	activeTile: false,
	sorter: false,
	scrollers: {},

	initialize: function() {
		console.log("Project Maya initted.");
		this.db = new Db(this);
		this.phrase = new Phrase({
			'onCleared': function() {
					$('play').removeClass("active");
					$('playGoogle').removeClass("active");
					console.log("cleared!");
				},
			'onFilled' : function() {
					console.log("filled!");
					$('play').addClass("inactive");
					$('playGoogle').addClass("inactive");
			},
			'onEnded' : function() {
					//$('')
			}
		});

		$('app').removeClass('loading').addClass('loaded').addEvents({
			// change between playaround and sentence mode
			'mouseup:relay(#rightpanel button[data-playaround-switch])' : function(evt) {
					var sentence = this.phrase.playAroundMode = evt.target.get('data-playaround-switch') === "true";
					$$("#rightpanel button[data-playaround-switch]").removeClass("active");
					evt.target.addClass('active');
					
					var method = !sentence ? 'addClass' : 'removeClass';
					$$('#rightpanel #playaround-buttons')[method]('visible');
				}.bind(this),
			// generic panel toggler
			'mouseup:relay(#rightpanel button[data-panel-switch])' : function(evt) {
					$$("#rightpanel .panel").removeClass("active");
					$$("#rightpanel #"+evt.target.get('data-panel-switch')).addClass("active");
				},
			// generic soundboard toggler
			'mouseup:relay(#rightpanel button[data-soundboard])' : function(evt) {
					this.showSoundboard(evt.target.get('data-soundboard'));
			}.bind(this),
			// generic title activator. Note the automatic disabling using :not()
			'mouseup:relay(.page.visible:not(.reordering) .soundboard.visible .tiles .tile:not(.disabled))': this.activateTile.bind(this),
			// handles play button
			'mouseup:relay(button#play)' : function() {
					this.phrase.play();
				}.bind(this),
			// handles play via google butotn
			'mouseup:relay(button#playGoogle)' : function() {
					this.phrase.playGoogle();
				}.bind(this),
			// handles clear button
			'mouseup:relay(button#clear)' : function() {
					this.phrase.clear().draw();
					$$(".page.visible .tiles .tile.inverted").removeClass("inverted");
				}.bind(this),
			// handles reorder button
			'mouseup:relay(button[data-reorder-switch])': this.handleSort.bind(this)
		});
	
		this.getSoundBoards();
	},

	databaseSetup: function() {
		this.showPage('databasesetup');
		$('query').set('text', 'Downloading database dump');
			new Request({
				url: './db.sqlite.sql',
				onComplete: function(response) {
					$("query").set('text', "Filling database with default words and pages");
					var progress = new ProgressBar({
						container: $('progress')					
					});
					var queries = response.split(";\n");
					this.db.processImport(0, queries, progress, function() {
						this.getSoundBoards();
						this.db.getSoundsForPage('Home', function(sounds) {
							this.injectSoundsIntoPage('Home', sounds);
						}.bind(this), function(err) { alert(err.message); });
					}.bind(this));
				}.bind(this)
			}).send();
	},

	getSoundBoards: function() {
		var injectPage = function(pageTitle, sounds) {
			this.injectSoundsIntoPage(pageTitle, sounds);
			if(pageTitle == 'Home') {
				this.showPage('Home');
				this.showSoundboard('Home');
			}
		}.bind(this);
		this.db.getSoundBoards(function(res) {
			var boardButtons = $$(".boardbuttons");
			var pageContainer = $("Home");
			for(i=0; i<res.rows.length; i++) {
				var page =  res.rows.item(i);
				console.log("Processing soundboard: ", res.rows.item(i));
				boardButtons.each(function(el) {
					el.adopt(new Element('button', {
					text: page.Title,
					'data-soundboard': page.Title,
					'data-page-id': page.ID_Page
					}));
				});

				new Element('div', {
					'class':'soundboard',
					'data-soundboard': page.Title,
					'data-page-id': page.ID_Page
				}).adopt(
					new Element("div.tiles")
				).inject(pageContainer);
				this.db.getSoundsForPage(page.Title, injectPage);
			}
		}.bind(this), function() { console.log("No pages defined."); this.databaseSetup(); }.bind(this));
	},

	injectSoundsIntoPage: function(soundboard, sounds) {
		if(!sounds.rows) {
			return;
		}
		console.log("Found audio : ", sounds, "to inject into soundboard: ",soundboard, "div.soundboard[data-soundboard='"+soundboard+"']");
		var tilesContainer = $$("div.soundboard[data-soundboard='"+soundboard+"'] .tiles")[0];
		var tpl = new Element('div.wrapper').adopt(
				new Element('div.tile', { 'data-wavefile': '' }).adopt(
					new Element('a.sprite').adopt(
						new Element('span', {text: '' })
						)
					)
				);
		for(var i=0; i< sounds.rows.length; i++) {
			var sound = sounds.rows.item(i);
			var curEl = tpl.clone(true,true);
			curEl.getFirst('div.tile').set({
				'data-ID_Audio' : sound.ID_Audio,
				'data-wavefile': sound.phrase
			});
			curEl.getFirst('div.tile a span').addClass(sound.icon).set('text', sound.label);
			curEl.inject(tilesContainer);
		}
	},

	showPage: function(page) {
		console.log("Showing page: ", page);
		$$(".page").removeClass('visible');
		$(page).addClass('visible');
		this.activePage = page;
	},

	showSoundboard: function(sb) {
		console.log("Showing soundboard: ", sb);
		$$(".soundboard").removeClass('visible');
		$$("div.soundboard[data-soundboard='"+sb+"']").addClass('visible');
		this.activeSoundboard = sb;
	},

	activateTile: function(evt) {
		var el = evt.target;
		if(evt.changedTouches) {
			el = $(document.elementFromPoint(evt.changedTouches[0].pageX,evt.changedTouches[0].pageY));
		}
		var target = el.hasClass('tile') ? el : el.getParent('.tile');
		if(el.hasClass('wrapper')) { target= el.getElement('.tile'); }
		if(this.phrase.playAroundMode === true) {
			$(this.activePage).getElements('.tile.inverted').removeClass('inverted');
		}
		if(!target.hasClass('inverted')) {
			target.addClass('inverted');
			this.phrase.addWord(target).draw();
		} else {
			target.removeClass('inverted');
			this.phrase.removeWord(target).draw();
		}
	},


	handleSort: function(evt) {

		var init =  evt.target.get('data-reorder-switch') === 'true';
		if(!init && this.sorter) {
			console.log("Detaching!");
			var newOrder = this.sorter.serialize(false, function(element, index){
				return { soundboard: element.getParent('.soundboard').get('data-soundboard'), ID_Audio: element.getElement('.tile').get('data-ID_Audio'), sort_index: index };
			    //return element.getProperty('id').replace('item_','') + '=' + index;
			});
			newOrder.each(this.db.updateSortOrder.bind(this.db));

			this.sorter.detach();
			$(this.activePage).removeClass('reordering');
			$$(".soundboard .tiles .tile").removeClass("wobble");
		} else {
			console.log("Initializing sorter for " + $$('div.soundboard.visible'));
			$$(".soundboard .tiles .tile").removeClass("inverted").addClass("wobble");

			this.sorter = new Sortables($$("div.soundboard .tiles"), {
				clone:true ,
				constrain: true,
				droppables: [$$("div.soundboard"), $$("button.soundboard")]
			}).addEvents({
				start: function(el, clone) {
					clone.addClass("ghost").removeClass('wobble');
				},
				enter: function(el, droppale) {
					console.log("Drag enter! ", el, droppable);
					var board = droppable.get('data-soundboard');
					if(board) {	this.showSoundboard(board);	}
				},
				drop: function(element, droppable){
			      if (droppable) droppable.setStyle('border', '3px solid yellow');
			    }
			});
	      
	      $(this.activePage).addClass('reordering');
	  }
	},

   /**
    * Shuffle the actual DOM order of these elements.
	*/
	rearrangeDOM: function(elements, parent, newOrder){
		var rearranged = [];
		parent = $(parent);
		newOrder.each(function(index){    //move each element and store the new default order
			if(elements[index]) {
			rearranged.push(elements[index].inject(parent));
			}
		}, this);
		elements = $$(rearranged);
	}
});

/**
 * Simple container for phrases.
 * adds words, removes words, clears list and plays.
 */
Phrase = new Class({

	Implements: [Options, Events],
	list : false,
	words: [],
	sounds: {},
	playIndex: 0,
	playAroundMode: true,

	initialize: function(options) {
		this.setOptions(options);
		this.list = $('wordslist');
		this.draw();
	},

	addWord: function(el) {
		if(this.playAroundMode) {
			this.clear();
		}
		this.words.push(el);
		if(this.words.length == 1) {
			this.fireEvent('filled');
		}
		console.log("added word: ", el);
		return this.draw();
	},

	removeWord: function(el) {
		this.words.erase(el);
		if(this.words.length === 0) { this.fireEvent('cleared'); }
		return this;
	},

	clear: function() {
		this.words = [];
		this.playIndex = 0;
		this.fireEvent('cleared');
		return this;
	},

	draw: function() {
		this.list.set('html', '');
		if(this.words.length === 0) {
			new Element("li", {text:'touch a tile!'}).inject(this.list);
		} else {
			for(i=0; i<this.words.length; i++) {
				new Element("li", {text: this.words[i].getElement('span').get('text')}).inject(this.list);
			}
		}
		return this.playAroundMode ? this.play() : this;
	},

	play: function(index) {
		index = index || 0;
		if(index < this.words.length) {
			this.playIndex = index;
			var wavName = this.words[index].getAttribute('data-wavefile');
			if(!this.sounds[wavName]) {
				this.sounds[wavName] = new Element('audio', { 'data-id': wavName,  src: 'audio/'+wavName+'.wav'});
			}
			console.log("Playgin! ", wavName);
			this.sounds[wavName].play();
			setTimeout(this.playNext.bind(this), 200);
		}
		return this;
	},

	playGoogle: function() {
		words = [];
		for(var i=0; i< this.words.length; i++) {
			words.push(this.words[i].getElement('span').get('text'));
		}
		var sentence = words.join(' ');
		var wavName = encodeURIComponent(sentence);
		if(!this.sounds[wavName]) {
				this.sounds[wavName] = new Element('audio', { 'data-id': wavName,  src: 'audio/services.php?action=translategoogle&tl=en&q='+wavName});
			}
		console.log("Playgin! ", wavName);
		this.sounds[wavName].play();
		this.clear().draw();

	},


	playNext: function() {
		var wavName = this.words[this.playIndex].getAttribute('data-wavefile');
		if(this.sounds[wavName].playing || (this.playIndex + 1 < this.words.length && !this.sounds[wavName].ended)) {
			return setTimeout(this.playNext.bind(this), 200);
		}

		if(this.playIndex +1 < this.words.length) {
			console.log("Playing next!, current is done: ", wavName, this.playIndex, this.words);
			this.play(this.playIndex + 1);
		}
	}
});



/**
 * Ultra simple db class that's DRY compatible
 */
Db = new Class({
	Implements: [Options, Events],

	initialize: function(parent) {
		this.parent = parent;
		this.db = window.openDatabase("mayawords", "1.0", "maya", 1000000);
		this.sessionID = new Date().getTime() / 1000;
	},
	
	/**
	* For debug purposes.
	* Wipes the audio and pages tables. Initialize using js console or bookmarklet with
	* window.projectmaya.db.reInit()
	*/
	reInit: function() {
		this.query("drop table `audio`;", [], function(){});
		this.query("drop table `page`;", [], function(){});
		setTimeout(function() {
			window.location.reload();
		}, 2000);
	},

	query: function(query, parameters, oncomplete, onerror) {
		console.log("Executing query: ", query, parameters);
		if(!onerror) onerror = this.defaultError;
		if(!oncomplete) oncomplete = this.defaultComplete;
		this.db.transaction(function(conn) {
			conn.executeSql(query, parameters, function(tx, result) {
				oncomplete(result.rows.length == 1 ? result.rows.item(0) : result);
			}, onerror);
		});
	},

	defaultError: function(tx, err) {
		console.log("Query error!", err);
	},

	defaultComplete: function(tx, res) {
		console.log("Query OK", res);
	},

	processImport: function(i, queries, progress, complete) {
		if(i < queries.length) {
			console.log("Setting progress to : ", parseInt((100 / queries.length) * i, 10));
			progress.set(parseInt((100 / queries.length) * i, 10));
			this.query(queries[i]+';', [],	function(tx, result) {
				this.processImport(i +1, queries, progress, complete);
			}.bind(this),					function(err) {
				console.log("Query error in ", queries[i], err.message);
				this.processImport(i +1, queries, progress, complete);
			}.bind(this));
		} else {
			$('query').set('text', 'done');
			complete();
		}
	},

	/**
	 * Build an sqlite3 compatible case when statement for updating multiple rows with one query.
	 */
	updateSortOrder: function(elements) {
		var cw = [], ids=[];
		elements.each(function(el) {
			cw.push('WHEN ' +el.ID_Audio + ' THEN '+ el.sort_index );
			ids.push(el.ID_Audio);
		}.bind(this));
		this.query('update audio set sort_index = CASE ID_Audio '+cw.join(' ')+' END WHERE ID_Audio IN ('+ids.join(',')+');', []);
	},

	// only in chrome for now :()
	getSound: function(sound, complete, error) {
		this.query("select base64wave from audio where phrase = ?", [sound], complete, error);
	},

	getSoundBoards: function(complete, error) {
		this.query("select * from page order by sortorder", [], complete, error);
	},

	getSoundsForPage: function(pg, complete, error) {
		this.query("select ID_Audio, icon, background, sort_index, label, phrase from audio left join page on audio.ID_Page = page.ID_Page where page.Title = ? order by audio.sort_index, page.ID_Page asc", [pg], function(result) {
			complete(pg, result);
		}, error);
	}

});




/**
  * Generic touch handler, makes mouse events fire for touch events, and
  * prevents both touchend and mouseup firing right after eachother on tablets.
  * Mouseup is important, since we use that instead of mousedown to get the target element from e.changedTouches
  */
function touchHandler(event) {
    var touches = event.changedTouches, first = touches[0], type = "";
    switch(event.type) {
         case "touchstart":	type = "mousedown"; break;
         case "touchmove":	type="mousemove";	break;
         case "touchcancel":
         case "touchend":	type="mouseup";		break;
         case 'mouseup': if(!!('ontouchstart' in window)) { evt.preventDefault();  } break; // do not fire mouseup twice on tablets
         default: return;
    }
    var simulatedEvent = document.createEvent("MouseEvent");
    simulatedEvent.initMouseEvent(type, true, true, window, 1, first.screenX, first.screenY, first.clientX, first.clientY, false, false, false, false, 0, null);
    first.target.dispatchEvent(simulatedEvent);
}
 
window.addEvent('domready', function() {
    document.addEventListener("touchstart", touchHandler, false);
    document.addEventListener("touchmove", touchHandler, false);
    document.addEventListener("touchend", touchHandler, false);
    document.addEventListener("touchcancel", touchHandler, false);
});